@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2012  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2014  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "../config.h"
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,26 +42,34 @@
 #include "copyright.h"
 
 
-static int cw_process_option(int opt, const char *optarg, cw_config_t *config, const char *argv0);
-static void cw_print_usage(const char *argv0);
+static int cw_process_option(int opt, const char *optarg, cw_config_t *config);
+static void cw_print_usage(const char *program_name);
 
 
 /*---------------------------------------------------------------------*/
 /*  Command line helpers                                               */
 /*---------------------------------------------------------------------*/
 
-/*
- * program_basename()
- *
- * Return the program's base name from the given argv[0].
- */
-const char *
-program_basename (const char *argv0)
-{
-  const char *base;
 
-  base = strrchr (argv0, '/');
-  return base ? base + 1 : argv0;
+
+
+
+/**
+   \brief Return the program's base name from the given argv0
+
+   Function returns pointer to substring in argv[0], so I guess that the
+   pointer is owned by environment (?).
+   Since there is always some non-NULL argv[0], the function always returns
+   non-NULL pointer.
+
+   \param argv0 - first argument to the program, argv[0]
+
+   \return program's name
+*/
+const char *cw_program_basename(const char *argv0)
+{
+	const char *base = strrchr(argv0, '/');
+	return base ? base + 1 : argv0;
 }
 
 
@@ -277,78 +285,81 @@ get_optind (void)
 
 
 
-void cw_print_help(const char *argv0, cw_config_t *config)
+void cw_print_help(cw_config_t *config)
 {
 	/* int format = has_longopts() */
-	fprintf(stderr, _("Usage: %s [options...]\n"), argv0);
+	fprintf(stderr, _("Usage: %s [options...]\n"), config->program_name);
 
 	if (!has_longopts()) {
-		fprintf(stderr, _("Long format of options is not supported on your system\n\n"));
+		fprintf(stderr, "%s", _("Long format of options is not supported on your system\n\n"));
 	}
 
-	fprintf(stderr, _("Audio system options:\n"));
-	fprintf(stderr, _("  -s, --system=SYSTEM\n"));
-	fprintf(stderr, _("        generate sound using SYSTEM audio system\n"));
-	fprintf(stderr, _("        SYSTEM: {console|oss|alsa|soundcard}\n"));
-	fprintf(stderr, _("        'console': use system console/buzzer\n"));
-	fprintf(stderr, _("               this output may require root privileges\n"));
-	fprintf(stderr, _("        'oss': use OSS output\n"));
-	fprintf(stderr, _("        'alsa' use ALSA output\n"));
-	fprintf(stderr, _("        'soundcard': use either OSS or ALSA\n"));
-	fprintf(stderr, _("        default sound system: 'oss'\n\n"));
-	fprintf(stderr, _("  -d, --device=DEVICE\n"));
-	fprintf(stderr, _("        use DEVICE as output device instead of default one;\n"));
-	fprintf(stderr, _("        optional for {console|alsa|oss};\n"));
-	fprintf(stderr, _("        default devices are:\n"));
-	fprintf(stderr, _("        'console': \"%s\"\n"), CW_DEFAULT_CONSOLE_DEVICE);
-	fprintf(stderr, _("        'oss': \"%s\"\n"), CW_DEFAULT_OSS_DEVICE);
-	fprintf(stderr, _("        'alsa': \"%s\"\n\n"), CW_DEFAULT_ALSA_DEVICE);
+	fprintf(stderr, "%s", _("Audio system options:\n"));
+	fprintf(stderr, "%s", _("  -s, --system=SYSTEM\n"));
+	fprintf(stderr, "%s", _("        generate sound using SYSTEM audio system\n"));
+	fprintf(stderr, "%s", _("        SYSTEM: {null|console|oss|alsa|pulseaudio|soundcard}\n"));
+	fprintf(stderr, "%s", _("        'null': don't use any sound output\n"));
+	fprintf(stderr, "%s", _("        'console': use system console/buzzer\n"));
+	fprintf(stderr, "%s", _("               this output may require root privileges\n"));
+	fprintf(stderr, "%s", _("        'oss': use OSS output\n"));
+	fprintf(stderr, "%s", _("        'alsa' use ALSA output\n"));
+	fprintf(stderr, "%s", _("        'pulseaudio' use PulseAudio output\n"));
+	fprintf(stderr, "%s", _("        'soundcard': use either PulseAudio, OSS or ALSA\n"));
+	fprintf(stderr, "%s", _("        default sound system: 'pulseaudio'->'oss'->'alsa'\n\n"));
+	fprintf(stderr, "%s", _("  -d, --device=DEVICE\n"));
+	fprintf(stderr, "%s", _("        use DEVICE as output device instead of default one;\n"));
+	fprintf(stderr, "%s", _("        optional for {console|oss|alsa|pulseaudio};\n"));
+	fprintf(stderr, "%s", _("        default devices are:\n"));
+	fprintf(stderr,       _("        'console': \"%s\"\n"), CW_DEFAULT_CONSOLE_DEVICE);
+	fprintf(stderr,       _("        'oss': \"%s\"\n"), CW_DEFAULT_OSS_DEVICE);
+	fprintf(stderr,       _("        'alsa': \"%s\"\n"), CW_DEFAULT_ALSA_DEVICE);
+	fprintf(stderr,       _("        'pulseaudio': %s\n\n"), CW_DEFAULT_PA_DEVICE);
 
-	fprintf(stderr, _("Sending options:\n"));
+	fprintf(stderr, "%s", _("Sending options:\n"));
 
-	fprintf(stderr, _("  -w, --wpm=WPM          set initial words per minute\n"));
-	fprintf(stderr, _("                         valid values: %d - %d\n"), CW_SPEED_MIN, CW_SPEED_MAX);
-	fprintf(stderr, _("                         default value: %d\n"), CW_SPEED_INITIAL);
-	fprintf(stderr, _("  -t, --tone=HZ          set initial tone to HZ\n"));
-	fprintf(stderr, _("                         valid values: %d - %d\n"), CW_FREQUENCY_MIN, CW_FREQUENCY_MAX);
-	fprintf(stderr, _("                         default value: %d\n"), CW_FREQUENCY_INITIAL);
-	fprintf(stderr, _("  -v, --volume=PERCENT   set initial volume to PERCENT\n"));
-	fprintf(stderr, _("                         valid values: %d - %d\n"), CW_VOLUME_MIN, CW_VOLUME_MAX);
-	fprintf(stderr, _("                         default value: %d\n"), CW_VOLUME_INITIAL);
+	fprintf(stderr, "%s", _("  -w, --wpm=WPM          set initial words per minute\n"));
+	fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_SPEED_MIN, CW_SPEED_MAX);
+	fprintf(stderr,       _("                         default value: %d\n"), CW_SPEED_INITIAL);
+	fprintf(stderr, "%s", _("  -t, --tone=HZ          set initial tone to HZ\n"));
+	fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_FREQUENCY_MIN, CW_FREQUENCY_MAX);
+	fprintf(stderr,       _("                         default value: %d\n"), CW_FREQUENCY_INITIAL);
+	fprintf(stderr, "%s", _("  -v, --volume=PERCENT   set initial volume to PERCENT\n"));
+	fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_VOLUME_MIN, CW_VOLUME_MAX);
+	fprintf(stderr,       _("                         default value: %d\n"), CW_VOLUME_INITIAL);
 
-	fprintf(stderr, _("Dot/dash options:\n"));
-	fprintf(stderr, _("  -g, --gap=GAP          set extra gap between letters\n"));
-	fprintf(stderr, _("                         valid values: %d - %d\n"), CW_GAP_MIN, CW_GAP_MAX);
-	fprintf(stderr, _("                         default value: %d\n"), CW_GAP_INITIAL);
-	fprintf(stderr, _("  -k, --weighting=WEIGHT set weighting to WEIGHT\n"));
-	fprintf(stderr, _("                         valid values: %d - %d\n"), CW_WEIGHTING_MIN, CW_WEIGHTING_MAX);
-	fprintf(stderr, _("                         default value: %d\n"), CW_WEIGHTING_INITIAL);
+	fprintf(stderr, "%s", _("Dot/dash options:\n"));
+	fprintf(stderr, "%s", _("  -g, --gap=GAP          set extra gap between letters\n"));
+	fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_GAP_MIN, CW_GAP_MAX);
+	fprintf(stderr,       _("                         default value: %d\n"), CW_GAP_INITIAL);
+	fprintf(stderr, "%s", _("  -k, --weighting=WEIGHT set weighting to WEIGHT\n"));
+	fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_WEIGHTING_MIN, CW_WEIGHTING_MAX);
+	fprintf(stderr,       _("                         default value: %d\n"), CW_WEIGHTING_INITIAL);
 
-	fprintf(stderr, _("Other options:\n"));
+	fprintf(stderr, "%s",     _("Other options:\n"));
 	if (config->is_cw) {
-		fprintf(stderr, _("  -e, --noecho           disable sending echo to stdout\n"));
-		fprintf(stderr, _("  -m, --nomessages       disable writing messages to stderr\n"));
-		fprintf(stderr, _("  -c, --nocommands       disable executing embedded commands\n"));
-		fprintf(stderr, _("  -o, --nocombinations   disallow [...] combinations\n"));
-		fprintf(stderr, _("  -p, --nocomments       disallow {...} comments\n"));
+		fprintf(stderr, "%s", _("  -e, --noecho           disable sending echo to stdout\n"));
+		fprintf(stderr, "%s", _("  -m, --nomessages       disable writing messages to stderr\n"));
+		fprintf(stderr, "%s", _("  -c, --nocommands       disable executing embedded commands\n"));
+		fprintf(stderr, "%s", _("  -o, --nocombinations   disallow [...] combinations\n"));
+		fprintf(stderr, "%s", _("  -p, --nocomments       disallow {...} comments\n"));
 	}
 	if (config->has_practice_time) {
-		fprintf(stderr, _("  -T, --time=TIME        set initial practice time (in minutes)\n"));
-		fprintf(stderr, _("                         valid values: %d - %d\n"), CW_PRACTICE_TIME_MIN, CW_PRACTICE_TIME_MAX);
-		fprintf(stderr, _("                         default value: %d\n"), CW_PRACTICE_TIME_INITIAL);
+		fprintf(stderr, "%s", _("  -T, --time=TIME        set initial practice time (in minutes)\n"));
+		fprintf(stderr,       _("                         valid values: %d - %d\n"), CW_PRACTICE_TIME_MIN, CW_PRACTICE_TIME_MAX);
+		fprintf(stderr,       _("                         default value: %d\n"), CW_PRACTICE_TIME_INITIAL);
 	}
 	if (config->has_infile) {
-		fprintf(stderr, _("  -f, --infile=FILE      read practice words from FILE\n"));
+		fprintf(stderr, "%s", _("  -f, --infile=FILE      read practice words from FILE\n"));
 	}
 	if (config->has_outfile) {
-		fprintf(stderr, _("  -F, --outfile=FILE     write current practice words to FILE\n"));
+		fprintf(stderr, "%s", _("  -F, --outfile=FILE     write current practice words to FILE\n"));
 	}
 	if (config->is_cw) {
-		fprintf(stderr, _("                         default file: stdin\n"));
+		fprintf(stderr, "%s", _("                         default file: stdin\n"));
 	}
 	fprintf(stderr, "\n");
-	fprintf(stderr, _("  -h, --help             print this message\n"));
-	fprintf(stderr, _("  -V, --version          print version information\n\n"));
+	fprintf(stderr, "%s", _("  -h, --help             print this message\n"));
+	fprintf(stderr, "%s", _("  -V, --version          print version information\n\n"));
 
 	return;
 }
@@ -359,20 +370,18 @@ void cw_print_help(const char *argv0, cw_config_t *config)
 
 int cw_process_argv(int argc, char *const argv[], const char *options, cw_config_t *config)
 {
-	const char *argv0 = program_basename(argv[0]);
-
 	int option;
 	char *argument;
 
 	while (get_option(argc, argv, options, &option, &argument)) {
-		if (!cw_process_option(option, argument, config, argv0)) {
+		if (!cw_process_option(option, argument, config)) {
 			return CW_FAILURE;
 		}
 	}
 
 	if (get_optind() != argc) {
-		fprintf(stderr, "libcw: expected argument after options\n");
-		cw_print_usage(argv0);
+		fprintf(stderr, "%s: expected argument after options\n", config->program_name);
+		cw_print_usage(config->program_name);
 		return CW_FAILURE;
 	} else {
 		return CW_SUCCESS;
@@ -383,11 +392,15 @@ int cw_process_argv(int argc, char *const argv[], const char *options, cw_config
 
 
 
-int cw_process_option(int opt, const char *optarg, cw_config_t *config, const char *argv0)
+int cw_process_option(int opt, const char *optarg, cw_config_t *config)
 {
 	switch (opt) {
 	case 's':
-		if (!strcmp(optarg, "alsa")
+		if (!strcmp(optarg, "null")
+		    || !strcmp(optarg, "n")) {
+
+			config->audio_system = CW_AUDIO_NULL;
+		} else if (!strcmp(optarg, "alsa")
 		    || !strcmp(optarg, "a")) {
 
 			config->audio_system = CW_AUDIO_ALSA;
@@ -395,6 +408,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 			   || !strcmp(optarg, "o")) {
 
 			config->audio_system = CW_AUDIO_OSS;
+		} else if (!strcmp(optarg, "pulseaudio")
+			   || !strcmp(optarg, "p")) {
+
+			config->audio_system = CW_AUDIO_PA;
 		} else if (!strcmp(optarg, "console")
 			   || !strcmp(optarg, "c")) {
 
@@ -405,27 +422,27 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 			config->audio_system = CW_AUDIO_SOUNDCARD;
 		} else {
-			fprintf(stderr, "libcw: invalid audio system (option 's'): %s\n", optarg);
+			fprintf(stderr, "%s: invalid audio system (option 's'): %s\n", config->program_name, optarg);
 			return CW_FAILURE;
 		}
 		break;
 
 	case 'd':
-		// fprintf(stderr, "libcw: d:%s\n", optarg);
+		// fprintf(stderr, "%s: d:%s\n", config->program_name, optarg);
 		if (optarg && strlen(optarg)) {
 			config->audio_device = strdup(optarg);
 		} else {
-			fprintf(stderr, "libcw: no device specified for option -d\n");
+			fprintf(stderr, "%s: no device specified for option -d\n", config->program_name);
 			return CW_FAILURE;
 		}
 		break;
 
 	case 'w':
 		{
-			// fprintf(stderr, "libcw: w:%s\n", optarg);
+			// fprintf(stderr, "%s: w:%s\n", config->program_name, optarg);
 			int speed = atoi(optarg);
 			if (speed < CW_SPEED_MIN || speed > CW_SPEED_MAX) {
-				fprintf(stderr, "libcw: speed out of range: %d\n", speed);
+				fprintf(stderr, "%s: speed out of range: %d\n", config->program_name, speed);
 				return CW_FAILURE;
 			} else {
 				config->send_speed = speed;
@@ -435,10 +452,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 	case 't':
 		{
-			// fprintf(stderr, "libcw: t:%s\n", optarg);
+			// fprintf(stderr, "%s: t:%s\n", config->program_name, optarg);
 			int frequency = atoi(optarg);
 			if (frequency < CW_FREQUENCY_MIN || frequency > CW_FREQUENCY_MAX) {
-				fprintf(stderr, "libcw: frequency out of range: %d\n", frequency);
+				fprintf(stderr, "%s: frequency out of range: %d\n", config->program_name, frequency);
 				return CW_FAILURE;
 			} else {
 				config->frequency = frequency;
@@ -448,10 +465,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 	case 'v':
 		{
-			// fprintf(stderr, "libcw: v:%s\n", optarg);
+			// fprintf(stderr, "%s: v:%s\n", config->program_name, optarg);
 			int volume = atoi(optarg);
 			if (volume < CW_VOLUME_MIN || volume > CW_VOLUME_MAX) {
-				fprintf(stderr, "libcw: volume level out of range: %d\n", volume);
+				fprintf(stderr, "%s: volume level out of range: %d\n", config->program_name, volume);
 				return CW_FAILURE;
 			} else {
 				config->volume = volume;
@@ -461,10 +478,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 	case 'g':
 		{
-			// fprintf(stderr, "libcw: g:%s\n", optarg);
+			// fprintf(stderr, "%s: g:%s\n", config->program_name, optarg);
 			int gap = atoi(optarg);
 			if (gap < CW_GAP_MIN || gap > CW_GAP_MAX) {
-				fprintf(stderr, "libcw: gap out of range: %d\n", gap);
+				fprintf(stderr, "%s: gap out of range: %d\n", config->program_name, gap);
 				return CW_FAILURE;
 			} else {
 				config->gap = gap;
@@ -474,10 +491,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 	case 'k':
 		{
-			// fprintf(stderr, "libcw: k:%s\n", optarg);
+			// fprintf(stderr, "%s: k:%s\n", config->program_name, optarg);
 			int weighting = atoi(optarg);
 			if (weighting < CW_WEIGHTING_MIN || weighting > CW_WEIGHTING_MAX) {
-				fprintf(stderr, "libcw: weighting out of range: %d\n", weighting);
+				fprintf(stderr, "%s: weighting out of range: %d\n", config->program_name, weighting);
 				return CW_FAILURE;
 			} else {
 				config->weighting = weighting;
@@ -487,10 +504,10 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 	case 'T':
 		{
-			// fprintf(stderr, "libcw: T:%s\n", optarg);
+			// fprintf(stderr, "%s: T:%s\n", config->program_name, optarg);
 			int time = atoi(optarg);
 			if (time < 0) {
-				fprintf(stderr, "libcw: practice time is negative\n");
+				fprintf(stderr, "%s: practice time is negative\n", config->program_name);
 				return CW_FAILURE;
 			} else {
 				config->practice_time = time;
@@ -502,7 +519,7 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 		if (optarg && strlen(optarg)) {
 			config->input_file = strdup(optarg);
 		} else {
-			fprintf(stderr, "libcw: no input file specified for option -f\n");
+			fprintf(stderr, "%s: no input file specified for option -f\n", config->program_name);
 			return CW_FAILURE;
 		}
 		/* TODO: access() */
@@ -512,7 +529,7 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 		if (optarg && strlen(optarg)) {
 			config->output_file = strdup(optarg);
 		} else {
-			fprintf(stderr, "libcw: no output file specified for option -F\n");
+			fprintf(stderr, "%s: no output file specified for option -F\n", config->program_name);
 			return CW_FAILURE;
 		}
 		/* TODO: access() */
@@ -539,16 +556,15 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 		break;
 
 	case 'h':
-		cw_print_help(argv0, config);
+		cw_print_help(config);
 		exit(EXIT_SUCCESS);
-
 	case 'V':
-		fprintf(stderr, _("%s version %s\n"), argv0, PACKAGE_VERSION);
+		fprintf(stderr, _("%s version %s\n"), config->program_name, PACKAGE_VERSION);
 		fprintf(stderr, "%s\n", CW_COPYRIGHT);
 		exit(EXIT_SUCCESS);
 	case '?':
 	default: /* '?' */
-		cw_print_usage(argv0);
+		cw_print_usage(config->program_name);
 		return CW_FAILURE;
 	}
 
@@ -558,14 +574,12 @@ int cw_process_option(int opt, const char *optarg, cw_config_t *config, const ch
 
 
 
-void cw_print_usage(const char *argv0)
+void cw_print_usage(const char *program_name)
 {
 	const char *format = has_longopts()
 		? _("Try '%s --help' for more information.\n")
 		: _("Try '%s -h' for more information.\n");
 
-	fprintf(stderr, format, argv0);
+	fprintf(stderr, format, program_name);
 	return;
 }
-
-
