@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
- * Copyright (C) 2011-2012  Kamil Ignacak (acerion@wp.pl)
+ * Copyright (C) 2011-2014  Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@ extern "C" {
 #endif
 
 
-extern const char *program_basename(const char *argv0);
+extern const char *cw_program_basename(const char *argv0);
 extern void combine_arguments(const char *env_variable,
 			      int argc, char *const argv[],
 			      int *new_argc, char **new_argv[]);
