@@ -1,5 +1,5 @@
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
-// Copyright (C) 2011-2012  Kamil Ignacak (acerion@wp.pl)
+// Copyright (C) 2011-2014  Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
 // 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 //
 
-#include "../config.h"
+#include "config.h"
 
 #include <cerrno>
 #include <iostream>
@@ -125,6 +125,9 @@ const QString GAP_WHATSTHIS =
 Application *Application::libcw_user_application_instance = NULL;
 
 
+
+
+
 // Application()
 //
 // Class constructor.  Creates the application main window an GUI frame, and
@@ -148,9 +151,14 @@ Application::Application() : QMainWindow (0)
 	make_central_widget();
 
 	make_auxiliaries_end();
+
+	return;
 }
 
 
+
+
+
 //-----------------------------------------------------------------------
 //  Libcw keying event callback
 //-----------------------------------------------------------------------
@@ -164,22 +172,29 @@ Application::Application() : QMainWindow (0)
 // instance's receiver handler function.
 //
 // This function is called in signal handler context.
-void
-Application::libcw_keying_event_static (void *, int key_state)
+void Application::libcw_keying_event_static(void *arg, int key_state)
 {
-  const Application *application = libcw_user_application_instance;
+	const Application *app = libcw_user_application_instance;
 
-  // Notify the receiver of a libcw keying event only if there is a user
-  // instance that is actively using the library and in receive mode.  The
-  // receiver handler function cannot determine this for itself.
-  if (application && application->is_using_libcw_
-      && application->modeset_.is_receive ())
-    {
-      application->receiver_->handle_libcw_keying_event (key_state);
-    }
+	// Notify the receiver of a libcw keying event only if there
+	// is a user instance that is actively using the library and
+	// in receive mode.  The receiver handler function cannot
+	// determine this for itself.
+	if (app
+	    && app->is_using_libcw_
+	    && app->modeset_.is_receive()) {
+
+		struct timeval *t = (struct timeval *) arg;
+		app->receiver_->handle_libcw_keying_event(t, key_state);
+	}
+
+	return;
 }
 
 
+
+
+
 //-----------------------------------------------------------------------
 //  Qt event and slot handlers
 //-----------------------------------------------------------------------
@@ -190,29 +205,34 @@ Application::libcw_keying_event_static (void *, int key_state)
 void Application::about()
 {
 	QMessageBox::about(0, ABOUT_CAPTION, ABOUT_TEXT);
+
+	return;
 }
 
 
+
+
+
 // closeEvent()
 //
 // Event handler for window close.  Requests a confirmation if we happen to
 // be busy sending.
-void
-Application::closeEvent (QCloseEvent *event)
+void Application::closeEvent(QCloseEvent *event)
 {
-  bool is_closing = true;
+	bool is_closing = true;
 
-  if (is_using_libcw_)
-    {
-      is_closing =
-          QMessageBox::warning (this, _("Xcwcp"),
-                                _("Busy - are you sure?"),
-                                _("&Exit"), _("&Cancel"), 0, 0, 1) == 0;
-      if (is_closing)
-        stop ();
-    }
+	if (is_using_libcw_) {
+		is_closing = QMessageBox::warning(this, _("Xcwcp"),
+						  _("Busy - are you sure?"),
+						  _("&Exit"), _("&Cancel"), 0, 0, 1) == 0;
+		if (is_closing) {
+			stop();
+		}
+	}
 
-  is_closing ? event->accept () : event->ignore ();
+	is_closing ? event->accept() : event->ignore();
+
+	return;
 }
 
 
@@ -226,6 +246,8 @@ Application::closeEvent (QCloseEvent *event)
 void Application::startstop()
 {
 	play_ ? stop() : start();
+
+	return;
 }
 
 
@@ -254,7 +276,7 @@ void Application::start()
 			// Restore button's proper visual appearance
 			// after it has been pressed, but user hasn't
 			// confirmed starting playing in this instance.
-			this->startstop_button_->setDown(false);
+			//this->startstop_button_->setDown(false);
 			return;
 		}
 	}
@@ -292,7 +314,7 @@ void Application::start()
 	action->setIcon(stop_icon);
 	action->setText(_("Stop"));
 	action->setToolTip(_("Stop"));
-	startstop_button_->setDown(true);
+	//startstop_button_->setDown(true);
 	play_ = true;
 
 	display_->clear_status();
@@ -301,6 +323,8 @@ void Application::start()
 	// maximum library speed needs a 10ms timeout.
 	poll_timer_->setSingleShot(false);
 	poll_timer_->start(10);
+
+	return;
 }
 
 
@@ -340,10 +364,12 @@ void Application::stop()
 	action->setIcon(start_icon);
 	action->setText(_("Start"));
 	action->setToolTip(_("Start"));
-	startstop_button_->setDown(false);
+	//startstop_button_->setDown(false);
 	play_ = false;
 
 	display_->show_status(_("Ready"));
+
+	return;
 }
 
 
@@ -353,50 +379,60 @@ void Application::stop()
 // new_instance()
 //
 // Creates a new instance of the Xcwcp application.
-void
-Application::new_instance ()
+void Application::new_instance()
 {
-  Application *application = new Application ();
-  //application->setCaption (_("Xcwcp"));
-  application->show ();
+	Application *app = new Application();
+	//app->setCaption (_("Xcwcp"));
+	app->show();
+
+	return;
 }
 
 
+
+
+
 // clear()
 //
 // Clears the display window of this Xcwcp instance.
-void
-Application::clear ()
+void Application::clear()
 {
-  display_->clear ();
+	display_->clear();
+
+	return;
 }
 
 
+
+
+
 // sync_speed()
 //
 // Forces the tracked receive speed into synchronization with the speed
 // spin box if adaptive receive is activated.
-void
-Application::sync_speed ()
+void Application::sync_speed()
 {
-  if (is_using_libcw_)
-    {
-      if (adaptive_receive_->isChecked ())
-        {
-          // Force by unsetting adaptive receive, setting the receive speed,
-          // then resetting adaptive receive again.
-          cw_disable_adaptive_receive ();
-          if (!cw_set_receive_speed (speed_spin_->value ()))
-            {
-              perror ("cw_set_receive_speed");
-              abort ();
-            }
-          cw_enable_adaptive_receive ();
-        }
-    }
+	if (is_using_libcw_) {
+		if (adaptive_receive_->isChecked()) {
+			// Force by unsetting adaptive receive,
+			// setting the receive speed, then resetting
+			// adaptive receive again.
+			cw_disable_adaptive_receive();
+			if (!cw_set_receive_speed(speed_spin_->value())) {
+				perror("cw_set_receive_speed");
+				abort();
+			}
+			cw_enable_adaptive_receive();
+		}
+	}
+
+	return;
 }
 
 
+
+
+
 // speed_change()
 // frequency_change()
 // volume_change()
@@ -406,114 +442,129 @@ Application::sync_speed ()
 // necessary is to write the new values out to the CW library.  The one thing
 // we do do is to only change parameters when we are active (i.e. have
 // control of the CW library).
-void
-Application::speed_change ()
+void Application::speed_change()
 {
-  if (is_using_libcw_)
-    {
-      if (!cw_set_send_speed (speed_spin_->value ()))
-        {
-          perror ("cw_set_send_speed");
-          abort ();
-        }
-      if (!cw_get_adaptive_receive_state ())
-        {
-          if (!cw_set_receive_speed (speed_spin_->value ()))
-            {
-              perror ("cw_set_receive_speed");
-              abort ();
-            }
-        }
-    }
+	if (is_using_libcw_) {
+		if (!cw_set_send_speed(speed_spin_->value())) {
+			perror("cw_set_send_speed");
+			abort();
+		}
+		if (!cw_get_adaptive_receive_state()) {
+			if (!cw_set_receive_speed(speed_spin_->value())) {
+				perror("cw_set_receive_speed");
+				abort();
+			}
+		}
+	}
+
+	return;
 }
 
-void
-Application::frequency_change ()
+
+
+
+
+void Application::frequency_change()
 {
-  if (is_using_libcw_)
-    {
-      if (!cw_set_frequency (frequency_spin_->value ()))
-        {
-          perror ("cw_set_frequency");
-          abort ();
-        }
-    }
+	if (is_using_libcw_) {
+		if (!cw_set_frequency(frequency_spin_->value())) {
+			perror("cw_set_frequency");
+			abort();
+		}
+	}
+
+	return;
 }
 
-void
-Application::volume_change ()
+
+
+
+
+void Application::volume_change()
 {
-  if (is_using_libcw_)
-    {
-      if (!cw_set_volume (volume_spin_->value ()))
-        {
-          perror ("cw_set_volume");
-          abort ();
-        }
-    }
+	if (is_using_libcw_) {
+		if (!cw_set_volume(volume_spin_->value())) {
+			perror("cw_set_volume");
+			abort();
+		}
+	}
+
+	return;
 }
 
-void
-Application::gap_change ()
+
+
+
+
+void Application::gap_change()
 {
-  if (is_using_libcw_)
-    {
-      if (!cw_set_gap (gap_spin_->value ()))
-        {
-          perror ("cw_set_gap");
-          abort ();
-        }
-    }
+	if (is_using_libcw_) {
+		if (!cw_set_gap(gap_spin_->value())) {
+			perror("cw_set_gap");
+			abort();
+		}
+	}
+
+	return;
 }
 
 
+
+
+
 // mode_change()
 //
 // Handle a change of mode.  Synchronize mode and receive speed if moving to
 // a receive mode, then clear the sender and receiver and any pending tones.
-void
-Application::mode_change ()
+void Application::mode_change()
 {
-  // Get the mode to which mode we're changing.
-  const Mode *new_mode = modeset_.get (mode_combo_->currentIndex());
+	// Get the mode to which mode we're changing.
+	const Mode *new_mode = modeset_.get(mode_combo_->currentIndex());
 
-  // If this changes mode type, set the speed synchronization menu item state
-  // to enabled for receive mode, disabled otherwise.  And for tidiness, clear
-  // the display.
-  if (!new_mode->is_same_type_as (modeset_.get_current ()))
-    {
-	    sync_speed_->setEnabled(new_mode->is_receive());
-      display_->clear ();
-    }
+	// If this changes mode type, set the speed synchronization
+	// menu item state to enabled for receive mode, disabled
+	// otherwise.  And for tidiness, clear the display.
+	if (!new_mode->is_same_type_as(modeset_.get_current())) {
+		sync_speed_->setEnabled(new_mode->is_receive());
+		display_->clear();
+	}
 
-  // If the mode changed while we're busy, clear the sender and receiver.
-  if (is_using_libcw_)
-    {
-      sender_->clear ();
-      receiver_->clear ();
-    }
+	// If the mode changed while we're busy, clear the sender and receiver.
+	if (is_using_libcw_) {
+		sender_->clear();
+		receiver_->clear();
+	}
 
-  // Keep the ModeSet synchronized to mode_combo changes.
-  modeset_.set_current(mode_combo_->currentIndex());
+	// Keep the ModeSet synchronized to mode_combo changes.
+	modeset_.set_current(mode_combo_->currentIndex());
+
+	return;
 }
 
 
+
+
+
 // curtis_mode_b_change()
 //
 // Called whenever the user request a change of Curtis iambic mode.  The
 // function simply passes the Curtis mode on to the CW library if active,
 // and ignores the call if not.
-void
-Application::curtis_mode_b_change ()
+void Application::curtis_mode_b_change()
 {
-  if (is_using_libcw_)
-    {
-      curtis_mode_b_->isChecked () ? cw_enable_iambic_curtis_mode_b ()
-                                   : cw_disable_iambic_curtis_mode_b ();
-    }
+	if (is_using_libcw_) {
+		curtis_mode_b_->isChecked()
+			? cw_enable_iambic_curtis_mode_b()
+			: cw_disable_iambic_curtis_mode_b();
+	}
+
+	return;
 }
 
 
+
+
+
 // adaptive_receive_change()
 //
 // Called whenever the user request a change of adaptive receive status.  The
@@ -521,41 +572,41 @@ Application::curtis_mode_b_change ()
 // active, and if fixed speed receive is set, also sets the hard receive speed
 // to equal the send speed, otherwise, it restores the previous tracked receive
 // speed.
-void
-Application::adaptive_receive_change ()
+void Application::adaptive_receive_change()
 {
-  if (is_using_libcw_)
-    {
-      if (adaptive_receive_->isChecked ())
-        {
-          // If going to adaptive receive, first set the speed to the saved
-          // receive speed, then turn on adaptive receiving.
-          cw_disable_adaptive_receive ();
-          if (!cw_set_receive_speed (saved_receive_speed_))
-            {
-              perror ("cw_set_receive_speed");
-              abort ();
-            }
-          cw_enable_adaptive_receive ();
-        }
-      else
-        {
-          // If going to fixed receive, save the current adaptive receive
-          // speed so we can restore it later, then turn off adaptive receive,
-          // and set the speed to equal the send speed as shown on the speed
-          // spin box.
-          saved_receive_speed_ = cw_get_receive_speed ();
-          cw_disable_adaptive_receive ();
-          if (!cw_set_receive_speed (speed_spin_->value ()))
-            {
-              perror ("cw_set_receive_speed");
-              abort ();
-            }
-        }
-    }
+	if (is_using_libcw_) {
+		if (adaptive_receive_->isChecked()) {
+			// If going to adaptive receive, first set the
+			// speed to the saved receive speed, then turn
+			// on adaptive receiving.
+			cw_disable_adaptive_receive();
+			if (!cw_set_receive_speed(saved_receive_speed_)) {
+				perror("cw_set_receive_speed");
+				abort ();
+			}
+			cw_enable_adaptive_receive();
+		} else {
+			// If going to fixed receive, save the current
+			// adaptive receive speed so we can restore it
+			// later, then turn off adaptive receive, and
+			// set the speed to equal the send speed as
+			// shown on the speed spin box.
+			saved_receive_speed_ = cw_get_receive_speed();
+			cw_disable_adaptive_receive();
+			if (!cw_set_receive_speed(speed_spin_->value())) {
+				perror("cw_set_receive_speed");
+				abort();
+			}
+		}
+	}
+
+	return;
 }
 
 
+
+
+
 // fonts()
 //
 // Use a font dialog to allow selection of display font.
@@ -568,9 +619,14 @@ void Application::fonts()
 		QWidget *display_widget = display_->get_widget();
 		display_widget->setFont(font);
 	}
+
+	return;
 }
 
 
+
+
+
 // colors()
 //
 // Use a color dialog to allow selection of display color.
@@ -586,9 +642,14 @@ void Application::colors()
 
 		display_widget->setPalette(palette);
 	}
+
+	return;
 }
 
 
+
+
+
 //-----------------------------------------------------------------------
 //  Timer, keyboard and mouse events
 //-----------------------------------------------------------------------
@@ -598,60 +659,66 @@ void Application::colors()
 // Handle a timer event from the QTimer we set up on initialization.  This
 // timer is used for regular polling for sender tone queue low and completed
 // receive characters.
-void
-Application::poll_timer_event ()
+void Application::poll_timer_event()
 {
-  if (is_using_libcw_)
-    {
-      sender_->poll (modeset_.get_current ());
-      receiver_->poll (modeset_.get_current ());
-    }
+	if (is_using_libcw_) {
+		sender_->poll(modeset_.get_current());
+		receiver_->poll(modeset_.get_current());
+	}
+
+	return;
 }
 
 
+
+
+
 // key_event()
 //
 // Handle a key press event from the display widget.
-void
-Application::key_event (QKeyEvent *event)
+void Application::key_event(QKeyEvent *event)
 {
-  event->ignore ();
+	event->ignore();
 
-  // Special case Alt-M as a way to acquire focus in the mode combo widget.
-  // This was a workround applied to earlier releases, no longer required
-  // now that events are propagated correctly to the parent.
-  //if (event->state () & AltButton && event->key () == Qt::Key_M)
-  //  {
-  //    mode_combo_->setFocus ();
-  //    event->accept ();
-  //    return;
-  //  }
+	// Special case Alt-M as a way to acquire focus in the mode
+	// combo widget.  This was a workaround applied to earlier
+	// releases, no longer required now that events are propagated
+	// correctly to the parent.
+	//if (event->state () & AltButton && event->key () == Qt::Key_M)
+	//  {
+	//    mode_combo_->setFocus ();
+	//    event->accept ();
+	//    return;
+	//  }
 
-  // Pass the key event to the sender and the receiver.
-  if (is_using_libcw_)
-    {
-      sender_->handle_key_event (event, modeset_.get_current ());
-      receiver_->handle_key_event (event, modeset_.get_current (),
-                                   reverse_paddles_->isChecked ());
+	// Pass the key event to the sender and the receiver.
+	if (is_using_libcw_) {
+		sender_->handle_key_event(event, modeset_.get_current());
+		receiver_->handle_key_event(event, modeset_.get_current(),
+					     reverse_paddles_->isChecked());
+	}
 
-    }
+	return;
 }
 
 
+
+
+
 // mouse_event()
 //
 // Handle a mouse event from the display widget.
-void
-Application::mouse_event (QMouseEvent *event)
+void Application::mouse_event(QMouseEvent *event)
 {
-  event->ignore ();
+	event->ignore();
 
-  // Pass the mouse event to the receiver.  The sender isn't interested.
-  if (is_using_libcw_)
-    {
-      receiver_->handle_mouse_event (event, modeset_.get_current (),
-                                     reverse_paddles_->isChecked ());
-    }
+	// Pass the mouse event to the receiver.  The sender isn't interested.
+	if (is_using_libcw_) {
+		receiver_->handle_mouse_event(event, modeset_.get_current(),
+					      reverse_paddles_->isChecked());
+	}
+
+	return;
 }
 
 
@@ -660,7 +727,6 @@ Application::mouse_event (QMouseEvent *event)
 
 void Application::toggle_toolbar(void)
 {
-
 	if (toolbar->isVisible()) {
 		toolbar->hide();
 		toolbar_visibility_->setText("Show Toolbar");
@@ -668,6 +734,8 @@ void Application::toggle_toolbar(void)
 		toolbar->show();
 		toolbar_visibility_->setText("Hide Toolbar");
 	}
+
+	return;
 }
 
 
@@ -683,6 +751,7 @@ void Application::make_toolbar(void)
 	startstop_->setText(_("Start"));
 	startstop_->setToolTip(_("Start"));
 	startstop_->setWhatsThis(STARTSTOP_WHATSTHIS);
+	startstop_->setCheckable(false);
 	connect(startstop_, SIGNAL (triggered(bool)), this, SLOT (startstop()));
 
 
@@ -692,7 +761,7 @@ void Application::make_toolbar(void)
 	// invisible, but it's there.
 	startstop_button_ = new QToolButton(toolbar);
 	startstop_button_->setDefaultAction(startstop_);
-	startstop_button_->setCheckable(true);
+	startstop_button_->setCheckable(false);
 	toolbar->addWidget(startstop_button_);
 
 
@@ -708,7 +777,8 @@ void Application::make_toolbar(void)
 
 	QLabel *speed_label_ = new QLabel(_("Speed:"), 0, 0);
 	toolbar->addWidget(speed_label_);
-	speed_spin_ = new QSpinBox();
+
+	speed_spin_ = new QSpinBox(toolbar);
 	speed_spin_->setMinimum(CW_SPEED_MIN);
 	speed_spin_->setMaximum(CW_SPEED_MAX);
 	speed_spin_->setSingleStep(CW_SPEED_STEP);
@@ -725,7 +795,8 @@ void Application::make_toolbar(void)
 
 	QLabel *tone_label = new QLabel(_("Tone:"));
 	toolbar->addWidget(tone_label);
-	frequency_spin_ = new QSpinBox(0);
+
+	frequency_spin_ = new QSpinBox(toolbar);
 	frequency_spin_->setMinimum(CW_FREQUENCY_MIN);
 	frequency_spin_->setMaximum(CW_FREQUENCY_MAX);
 	frequency_spin_->setSingleStep(CW_FREQUENCY_STEP);
@@ -742,7 +813,8 @@ void Application::make_toolbar(void)
 
 	QLabel *volume_label = new QLabel(_("Volume:"), 0, 0);
 	toolbar->addWidget(volume_label);
-	volume_spin_ = new QSpinBox(0);
+
+	volume_spin_ = new QSpinBox(toolbar);
 	volume_spin_->setMinimum(CW_VOLUME_MIN);
 	volume_spin_->setMaximum(CW_VOLUME_MAX);
 	volume_spin_->setSingleStep(CW_VOLUME_STEP);
@@ -759,8 +831,8 @@ void Application::make_toolbar(void)
 
 	QLabel *gap_label = new QLabel(_("Gap:"), 0, 0);
 	toolbar->addWidget(gap_label);
-	gap_spin_ = new QSpinBox(0);
-	toolbar->addWidget(gap_spin_);
+
+	gap_spin_ = new QSpinBox(toolbar);
 	gap_spin_->setMinimum(CW_GAP_MIN);
 	gap_spin_->setMaximum(CW_GAP_MAX);
 	gap_spin_->setSingleStep(CW_GAP_STEP);
@@ -769,6 +841,7 @@ void Application::make_toolbar(void)
 	gap_spin_->setWhatsThis(GAP_WHATSTHIS);
 	gap_spin_->setValue(cw_get_gap());
 	connect(gap_spin_, SIGNAL (valueChanged(int)), SLOT (gap_change()));
+	toolbar->addWidget(gap_spin_);
 
 
 	// Finally for the toolbar, add whatsthis.
@@ -929,6 +1002,7 @@ void Application::make_help_menu(void)
 	connect(about_, SIGNAL(triggered(bool)), SLOT(about()));
 	help_->addAction(about_);
 
+	return;
 }
 
 
@@ -939,6 +1013,8 @@ void Application::make_central_widget(void)
 {
 	// this constructor calls setCentralWidget()
 	display_ = new Display(this, this->parentWidget());
+
+	return;
 }
 
 
@@ -955,17 +1031,11 @@ void Application::make_auxiliaries_begin(void)
 	saved_receive_speed_ = cw_get_receive_speed();
 	play_ = false;
 
-	// Register class handler as the CW library keying event callback. It's
-	// important here that we register the static handler, since once we have
-	// been into and out of 'C', all concept of 'this' is lost.  It's the job
-	// of the static handler to work out which class instance is using the CW
-	// library, and call the instance's libcw_keying_event() function.
-	cw_register_keying_callback(libcw_keying_event_static, NULL);
-
 	// Create a timer for polling send and receive.
 	poll_timer_ = new QTimer (this);
 	connect(poll_timer_, SIGNAL (timeout()), SLOT (poll_timer_event()));
 
+	return;
 }
 
 
@@ -978,12 +1048,60 @@ void Application::make_auxiliaries_end(void)
 	sender_ = new Sender(display_);
 	receiver_ = new Receiver(display_);
 
+	// Register class handler as the CW library keying event callback. It's
+	// important here that we register the static handler, since once we have
+	// been into and out of 'C', all concept of 'this' is lost.  It's the job
+	// of the static handler to work out which class instance is using the CW
+	// library, and call the instance's libcw_keying_event() function.
+	//
+	// The handler called back by libcw is important because it's
+	// used to send to libcw information about timings of events
+	// (key down and key up events).
+	//
+	// Without the callback the library can play sounds as key or
+	// paddles are pressed, but (since it doesn't receive timing
+	// parameters) it won't be able to identify entered Morse
+	// code.
+	cw_register_keying_callback(libcw_keying_event_static, &(receiver_->timer));
+
+	// The call above registered receiver_->timer as a generic
+	// argument to a callback. However, libcw needs to know when
+	// the argument happens to be of type 'struct timeval'. This
+	// is why we have this second call, explicitly passing
+	// receiver's timer to libcw.
+	cw_iambic_keyer_register_timer(&(receiver_->timer));
+
+	gettimeofday(&(receiver_->timer), NULL);
+	//fprintf(stderr, "time on aux config: %10ld : %10ld\n", receiver_->timer.tv_sec, receiver_->timer.tv_usec);
+
 	QString label("Output: ");
 	label += cw_generator_get_audio_system_label();
 	QLabel *sound_system = new QLabel(label);
 	statusBar()->addPermanentWidget(sound_system);
+
+	return;
+}
+
+
+
+
+
+void Application::check_audio_system(cw_config_t *config)
+{
+	if (config->audio_system == CW_AUDIO_ALSA
+	    && cw_is_pa_possible(NULL)) {
+
+		QMessageBox msgBox;
+		QString message1 = _("Selected audio system is ALSA, but audio on your system is handled by PulseAudio.");
+		QString message2 = _("Expect various problems.\n");
+		QString message3 = _("In this situation it is recommended to run %1 like this:\n" \
+				     "%2 -s p\n\n");
+		msgBox.setText(message1 + " " + message2 + message3.arg(config->program_name).arg(config->program_name));
+		msgBox.exec();
+	}
+
+	return;
 }
 
 
 }  // cw namespace
-
